<?php
namespace AustraliaPost\Extensions\Business;

use AustraliaPost\Core\Api\Abstract_Calculator;
use AustraliaPost\Core\Interface_Extender;
use AustraliaPost\Extensions\Business\API\Address_Validator;
use AustraliaPost\Extensions\Business\API\Business;
use AustraliaPost\Extensions\Business\Endpoints\Endpoints_Factory;
use AustraliaPost\Extensions\Business\Features\Tracking_Number_In_Orders_Page;
use AustraliaPost\Extensions\Business\Metaboxes\Labels\Label_Metabox;
use AustraliaPost\Extensions\Business\Migrations\Migrator;
use AustraliaPost\Extensions\Business\Orders\Order_Summary_Download_Page;
use AustraliaPost\Extensions\Business\Orders\Order_Summary_Settings;
use AustraliaPost\Extensions\Business\Outage\Outage_Alerts;
use AustraliaPost\Extensions\Business\Settings\Custom_Titles_Setting;
use AustraliaPost\Extensions\Business\Settings\Product_Shipping_Fields;
use AustraliaPost\Extensions\Business\Settings\Promotion_Widgets;
use AustraliaPost\Extensions\Business\Settings\Settings;
use WC_Order_Item_Shipping;

class Business_Adapter implements Interface_Extender {


	public function initiate()
	{
		// initiate endpoints
		Custom_Titles_Setting::get_instance();
		Endpoints_Factory::boot();
		Migrator::boot();
		Order_Summary_Settings::get_instance();
		Order_Summary_Download_Page::get_instance();
		Promotion_Widgets::get_instance();
		Settings::get_instance();
		// Address_Validator::get_instance();
		Outage_Alerts::get_instance();
		Product_Shipping_Fields::get_instance();
		Tracking_Number_In_Orders_Page::get_instance();
		add_action('woocommerce_before_order_object_save', [$this, 'move_packaging_details_to_order_object']);
	}

	public function settings() {
		return Settings::settings();
	}

	public function settings_keys() {
		return [
			'enabled_contract_mode',
			'api_password',
			'api_account_number',
		];
	}

	/**
	 * @param $settings
	 *
	 * @return Abstract_Calculator
	 */
	public function calculator($settings) {
		return new Business($settings);
	}

	public function should_use_calculator($settings)
	{
		return $settings['enabled_contract_mode'] === 'yes';
	}

	public function load_metaboxes()
	{
		Label_Metabox::get_instance();
	}

	public function is_enabled()
	{
		return true;
	}

	public function move_packaging_details_to_order_object( $order )
	{
		$shipping_items = array_values( $order->get_items( 'shipping' ) );
		if ( isset( $shipping_items[0] ) ) {
			/** @var WC_Order_Item_Shipping $shipping_line */
			$shipping_line = $shipping_items[0];
			$packaging_details = $shipping_line->get_meta( WPRUBY_PACKAGING_DETAILS_KEY );
			if ( $packaging_details ) {
				$shipping_line->delete_meta_data( WPRUBY_PACKAGING_DETAILS_KEY );
				$order->add_meta_data( WPRUBY_PACKAGING_DETAILS_KEY, $packaging_details );
				$shipping_line->save();
			}
		}
	}

}
